import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;

public class ActorsConverter {

	private static final String NAME_PREFIX = "N: ";
	private static final String MOVIE_PREFIX = "M: ";
	
	public static class Map extends
			Mapper<LongWritable, Text, LongWritable, Text> {
		
		private static final long INITIAL_OFFSET = 11770;
		private static final long OFFSET = 400000;
		private static final long ACTORS_OFFSET = 400000000;
		private static final String ACTORS_FILE = "actors_sample.list";
		
		private long mapPosition(long pos) {
			return pos / OFFSET;
		}
		
		private String constructOffsetString(long offset, String line) {
			return String.valueOf(offset) + "," + line; 
		}
		
		public void map(LongWritable key, Text value, Context context)
				throws IOException, InterruptedException {
			
			long position = key.get();
			if(position < INITIAL_OFFSET)
				return;
			
			FileSplit fileSplit = (FileSplit)context.getInputSplit();
			String fileName = fileSplit.getPath().getName();
			//context.write(new LongWritable(position), new Text(fileName));
			
			if(ACTORS_FILE.equals(fileName)) {
				position += ACTORS_OFFSET;
			}
			String str = value.toString();
			String [] splits = str.split("\t");
			if(splits.length > 1) {
				long mappedPosition = mapPosition(position);
				if(splits[0].length() > 0) {
					context.write(new LongWritable(mappedPosition), new Text(constructOffsetString(position, NAME_PREFIX + splits[0])));
					context.write(new LongWritable(mappedPosition + 1), new Text(constructOffsetString(position, NAME_PREFIX + splits[0])));
				}
				
				context.write(new LongWritable(mappedPosition), new Text(constructOffsetString(position + splits[0].length(), MOVIE_PREFIX + splits[splits.length - 1])));
			}
		}
	}

	public static class Reduce extends
			Reducer<LongWritable, Text, Text, Text> {

		public void reduce(LongWritable key, Iterable<Text> values,
				Context context) throws IOException, InterruptedException {
			//context.write(new Text("###"), values.iterator().next());
			int size = 0;
			List<OffsetTextPair> list = new LinkedList<OffsetTextPair>();
			for (Text val : values) {
				size++;
				list.add(new OffsetTextPair(val));
			}
			OffsetTextPair []offsetTextPairs = new OffsetTextPair[size];
			int idx = 0;
			for(OffsetTextPair val : list) {
				offsetTextPairs[idx++] = val;
			}
			Arrays.sort(offsetTextPairs);
			
			String lastName = null;
			for(int i = 0; i < offsetTextPairs.length; i++) {
				if(offsetTextPairs[i].getText().startsWith(NAME_PREFIX)) {
					lastName = offsetTextPairs[i].getText().substring(3);
				} else if(lastName != null && offsetTextPairs[i].getText().startsWith(MOVIE_PREFIX)) {
					context.write(new Text(lastName), new Text(offsetTextPairs[i].getText()));
				} else if(lastName != null) {
					throw new IllegalArgumentException("Found value which is niether name nor movie");
				} else {
					//context.write(new Text("###$$##" + offsetTextPairs[i].getText()), new Text(offsetTextPairs[i].getText()));
				}
			}
					
		}
	}

	public static void main(String[] args) throws Exception {
		Configuration conf = new Configuration();

		Job job = new Job(conf, "ActorsConverter");

		job.setOutputKeyClass(LongWritable.class);
		job.setOutputValueClass(Text.class);

		job.setMapperClass(Map.class);
		job.setReducerClass(Reduce.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);
		FileInputFormat.addInputPath(job, new Path(args[0]));
		FileOutputFormat.setOutputPath(job, new Path(args[1]));
        job.setJarByClass(ActorsConverter.class);
		job.waitForCompletion(true);
	}
}
